<?php

require_once ROOT_DIR . '/lib/JSON.php';

class Application {
    // Fields
    
    private $environment;
    private $services;
    
    // Constructor
    
    public function __construct($environment = '')
    {
        $this->environment = $environment;
        $this->services    = ServiceContainer::getInstance();
        
        $this->services->setApp($this);
    }
    
    // Methods
    
    public function getEnvironment()
    {
        return $this->environment;
    }
    
    public function config()
    {
        // Setup the services

        $this->services->registerService('exception_handler',  'ExceptionHandler');
        $this->services->registerService('config',             'Configuration');
        $this->services->registerService('logger',             'Logger');
        $this->services->registerService('db',                 'Database');
        $this->services->registerService('filesystem',         'Filesystem');
        $this->services->registerService('memory',             'Memory');
        $this->services->registerService('events',             'Events');
        $this->services->registerService('mailer',             'Mailer');
        $this->services->registerService('http',               'Http');
        $this->services->registerService('request',            'Request');
        $this->services->registerService('session',            'Session');
        $this->services->registerService('router',             'Router');
        $this->services->registerService('security',           'Security');
        $this->services->registerService('validation',         'Validation');
        $this->services->registerService('model_validation',   'ModelValidation');
        $this->services->registerService('models',             'Repository');
        $this->services->registerService('views',              'Repository');
        $this->services->registerService('controllers',        'Repository');
        $this->services->registerService('user',               'User');
        $this->services->registerService('guest',              'Guest');
        $this->services->registerService('auth',               'Authentication');
        $this->services->registerService('firewall',           'Firewall');
        $this->services->registerService('template_interface', 'TemplateInterface');
        $this->services->registerService('image_resizer',      'ImageResizer');
        $this->services->registerService('color_calculator',   'ColorCalculator');
    }
    
    public function run($request) {

        $response = null;

        $authorized = $this->services->get('firewall')->canAccessPath($request->getRoute());
        
        if($authorized){

            $router     = $this->services->get('router');
            $actionName = $router->getActionName($request->getRoute());

            if($actionName) {

                $response = $this->runAction($actionName);

            } else  {

                $controller = new Controller();
                $controller->setServiceContainer($this->services);
                $config      = $this->services->get('config');
                $appSettings = $config->data['appSettings'];

                if(empty($appSettings['installed']) && empty($config->data['superPass'])) {

                    $response = $controller->redirect('Install:index');
                } else {

                    $this->services->get('logger')->info('Route /' . $request->getRoute() . ' not found');
                    $response = $controller->render('error/404.html.php', array(), 'html', array(), 404);
                }
            }
        } else {
            
            $loginAction = $this->services->get('firewall')->getLoginAction();
            $loginPath   = '?' . $this->services->get('router')->getRoute($loginAction);
            $response = new Response('', array(array('Location', $loginPath)), 303);
        }
        
        $this->services->clean();
        return $response;
    }

    public function runAction($actionName) {

        $response = null;
        $action = $this->services->get('router')->getActionByName($actionName);
        $controller = $action['controller'];
        $actionName = $action['action'];
        $controller->setServiceContainer($this->services);

        try {
            
				$result = $controller->$actionName();
            if($result instanceof Response) {
                $response = $result;
            } else {
                $response = new Response($result);
            }
        } catch(Exception $e) {
            
            $response = $this->services->get('exception_handler')->createResponse($e);
        }

        return $response;
    }
}

?>
