<?php
class OperatorController extends Controller
{
    public function stayAliveAction()
    {
        UserModel::repo()->stayAlive($this->get('user')->getId());
        
        return $this->json(array('success' => true));
    }
    
    public function updateTypingStatusAction()
    {
        $request      = $this->get('request');
        $userId       = $this->get('user')->getId();
        $secondUserId = $request->postVar('secondUserId');
        
        if($userId)
        {
            $status = $request->postVar('status');
            UserModel::repo()->updateTypingStatus($userId, $secondUserId, $status == true);
            
            return $this->json(array('success' => true));
        }
        
        return $this->json(array('success' => false));
    }
    
    public function getTypingStatusAction()
    {
        $request = $this->get('request');
        $userId  = $this->get('user')->getId();
        $userIds = $request->postVar('ids');
        
        if($userId)
        {
            if(is_array($userIds))
            {
                $results = array();
                foreach($userIds as $id)
                {
                    $results[$id] = UserModel::repo()->getTypingStatus($userId, $id);
                }
                return $this->json(array('success' => true, 'results' => $results));
            }
        }
        
        return $this->json(array('success' => false));
    }
    
    public function updateAction()
    {
        $request     = $this->get('request');
        $security    = $this->get('security');
        $currentUser = $this->get('user');
        $id       = $request->postVar('id');
        $name     = $request->postVar('name');
        $mail     = $request->postVar('mail');
        $roles    = $request->postVar('roles');
        $image    = $request->postVar('image');
        $password = null;
        
        if($currentUser->getId() === $id || $currentUser->hasRole('ADMIN'))
        {
            $password = $request->postVar('password');
        }
        
        $errors = array();
        
        $user = UserModel::repo()->find($id);
        
        if($user && ($currentUser->getId() === $user->id || $currentUser->hasRole('ADMIN')))
        {
            if(!$currentUser->hasRole('ADMIN') && !empty($password))
            {
                $currPassword = $security->encodePassword($request->postVar('currPassword'));
                
                if($currPassword !== $user->password)
                {
                    $errors['password'] = 'Password: Current password is incorrect';
                }
            }
            
            if(count($errors) === 0)
            {
                $user->setRawData(compact('name', 'mail', 'password', 'roles', 'image'));
                $errors = $this->get('model_validation')->validateUser($user->getData(true), !empty($password));
                if(count($errors) === 0)
                {
                    if(!empty($password)) $user->password = $this->get('security')->encodePassword($user->password);
                    $user->save();
                    return $this->json(array('success' => true));
                }
            }
            
            return $this->json(array('success' => false, 'errors' => $errors));
        }
        else if($currentUser->hasRole('ADMIN') && !empty($password))
        {
            return $this->createAction();
        }
        
        return $this->json(array('success' => false, 'errors' => array('error' => 'User not found')));
    }
    
    public function getDefaultAvatarsAction()
    {
        $avatars  = UserModel::getDefaultAvatars();
        $template = $this->get('template_interface');
        $i = 0;
        
        foreach($avatars as $a)
        {
            $avatars[$i++] = $template->asset($a);
        }
        
        return $this->json(array('success' => true, 'avatars' => $avatars));
    }
    
    public function uploadAvatarAction()
    {
        $request = $this->get('request');
        $id   = $request->postVar('userId');
        $file = $request->file('avatar');
        $user = UserModel::repo()->find($id);
        
        if($user)
        {
            $filename = $this->get('security')->randomString(64) . '.' . $file['ext'];
            $inputPath  = $file['tmp_name'];
            $webPath    = 'upload/avatars/' . $filename;
            $outputPath = ROOT_DIR . '/../' . $webPath;
            $this->get('image_resizer')->resize($inputPath, $outputPath, UserModel::AVATAR_SIZE, UserModel::AVATAR_SIZE);
            $user->image = $request->getRootUrl() . $webPath;
            $user->save();
            return $this->json(array('success' => true, 'image' => $user->image), true);
        }
        return $this->json(array('success' => false, 'error' => 'User not found'));
    }

    public function listAction()
    {
        $users = UserModel::repo()->findBy(array('roles' => array('LIKE', '%OPERATOR%')));
        
        if($users)
        {
            foreach($users as $user)
            {
                unset($user->password);
            }
            return $this->json(array('success' => true, 'users' => $users));
        }
        return $this->json(array('success' => false));
    }
    
    public function createAction()
    {
        $request = $this->get('request');
        $name     = $request->postVar('name');
        $mail     = $request->postVar('mail');
        $password = $request->postVar('password');
        
        $roles = array('OPERATOR');
        $user = new UserModel(compact('name', 'mail', 'password', 'roles'));
        
        $errors = $this->get('model_validation')->validateUser($user->getData(true));
        
        if(count($errors) === 0)
        {
            $user->password = $this->get('security')->encodePassword($user->password);
            
            if($user->save())
            {
                return $this->json(array('success' => true, 'id' => $user->id));
            }
            
            $errors = array('N&atilde;o foi poss&iacute;vel salvar o usu&aacute;rio');
        }        
        return $this->json(array('success' => false, 'errors' => $errors));
    }

    public function deleteAction()
    {
        $request = $this->get('request');
        $id = $request->postVar('id');
        $user = UserModel::repo()->find($id);
        
        if($user)
        {
            $user->remove();
            return $this->json(array('success' => true));
        }
        return $this->json(array('success' => false));
    }

    public function getAction()
    {
        $request = $this->get('request');
        $id = $request->postVar('id');
        $user = UserModel::repo()->find($id);
        
        if($user)
        {
            unset($user->password);
            return $this->json(array('success' => true, 'user' => $user));
        }
        return $this->json(array('success' => false));
    }

    public function isOnlineAction()
    {
        $user = $this->get('guest');
        
        if(!$user->getId())
        {
            $config       = $this->get('config');
            $guestsOnline = UserModel::repo()->countGuestsOnline();
            
            if($guestsOnline >= $config->data['appSettings']['maxConnections'])
            {
                return $this->json(array('success' => false));
            }
        }
        
        return $this->json(array('success' => UserModel::repo()->isOperatorOnline()), array(), 'json', array(
            
            array('Cache-Control', 'no-cache, no-store, must-revalidate'),
            array('Pragma', 'no-cache'),
            array('Expires', '0')
        ));
    }

    public function getOnlineUsersAction()
    {
        return $this->json(array('success' => true, 'users' => UserModel::repo()->getAllOnline()));
    }
}
?>
