<?php

class ModelValidation extends Service
{
    private $validation;
    
    public function onRegister()
    {
        parent::onRegister();
        
        // -----
        
        $this->validation = $this->get('validation');
    }
    
    public function validateMessage($message)
    {
        return $this->clearEmpty(array(
        
            'from' => $this->validation->validate($message['from'], array(
            
                'O campo est&aacute; vazio'        => new NotBlankValidator(),
                'O valor informado n&atilde;o &eacute; um n&uacute;mero' => new NumberValidator()
            )),
            
            'to' => $this->validation->validate($message['to'], array(
            
                'O campo est&aacute; vazio'        => new NotBlankValidator(),
                'O valor informado n&atilde;o &eacute; um n&uacute;mero' => new NumberValidator()
            )),
            
            'body' => $this->validation->validate($message['body'], array(
            
                'O campo est&aacute; vazio' => new NotBlankValidator(),
            ))
        ));
    }
    
    public function validateUser($user, $checkPassword = true)
    {
        $data = array(
        
            'name' => $this->validation->validate($user['name'], array(
            
                'O campo est&aacute; vazio' => new NotBlankValidator()
            )),
            
            'mail' => $this->validation->validate($user['mail'], array(
            
                'O campo est&aacute; vazio'              => new NotBlankValidator(),
                'O campo n&atilde;o possui um e-mail v&aacute;lido' => new MailValidator()
            ))
        );
        
        if($checkPassword)
        {
            $data['password'] = $this->validation->validate($user['password'], array(
            
                'O campo est&aacute; vazio'     => new NotBlankValidator(),
                'O valor informado &eacute; muito curto' => new LengthValidator(6)
            ));
        }
        
        return $this->clearEmpty($data);
    }
    
    public function validateInstallConfig($config)
    {
        return $this->clearEmpty(array(
            
            'dbHost' => $this->validation->validate($config['dbHost'], array(
                
                'O campo est&aacute; vazio' => new NotBlankValidator()
            )),
            
            'dbPort' => $this->validation->validate($config['dbPort'], array(
                
                'O campo est&aacute; vazio'           => new NotBlankValidator(),
                'Value has to be a number' => new NumberValidator()
            )),
            
            'dbName' => $this->validation->validate($config['dbName'], array(
                
                'O campo est&aacute; vazio' => new NotBlankValidator()
            )),
            
            'dbUser' => $this->validation->validate($config['dbUser'], array(
                
                'O campo est&aacute; vazio' => new NotBlankValidator()
            )),
            /*
            'dbPassword' => $this->validation->validate($config['dbPassword'], array(
                
                'O campo est&aacute; vazio' => new NotBlankValidator()
            )),
            */
            'superUser' => $this->validation->validate($config['superUser'], array(
                
                'O campo est&aacute; vazio' => new NotBlankValidator()
            )),
            
            'superPass' => $this->validation->validate($config['superPass'], array(
                
                'O campo est&aacute; vazio' => new NotBlankValidator()
            )),
            
            'superPassRepeat' => $this->validation->validate($config['superPassRepeat'], array(
                
                'O campo est&aacute; vazio'      => new NotBlankValidator(),
                'Os campos n&atilde;o conferem' => new EqualsValidator($config['superPass'])
            )),
            
            'appSettings' => array(

                'contactMail' => $this->validation->validate($config['appSettings']['contactMail'], array(
                    
                    'O campo est&aacute; vazio'              => new NotBlankValidator(),
                    'O campo n&atilde;o possui um e-mail v&aacute;lido' => new MailValidator()
                ))
            )
        ));
    }
    
    public function validateContactData($data)
    {
        return $this->clearEmpty(array(
        
            'name' => $this->validation->validate($data['name'], array(
            
                'O campo est&aacute; vazio' => new NotBlankValidator()
            )),
            
            'mail' => $this->validation->validate($data['mail'], array(
            
                'O campo est&aacute; vazio'              => new NotBlankValidator(),
                'Value is not a valid e-mail' => new MailValidator()
            )),
            
            'question' => $this->validation->validate($data['question'], array(
            
                'O campo est&aacute; vazio'     => new NotBlankValidator(),
                'O valor informado &eacute; muito curto' => new LengthValidator(6)
            ))
        ));
    }
    
    public function validateLoginData($data)
    {
        return $this->clearEmpty(array(
        
            'name' => $this->validation->validate($data['name'], array(
            
                'O campo est&aacute; vazio' => new NotBlankValidator()
            )),
            
            'mail' => $this->validation->validate($data['mail'], array(
            
                'O campo est&aacute; vazio' => new NotBlankValidator(),
                'O campo n&atilde;o possui um e-mail v&aacute;lido' => new MailValidator()
            ))
        ));
    }

    public function validateConfig()
    {
        // No checks so far

        return array();
    }

    public function validateDb()
    {
        $errors = array('message' => 'A estrutura da base de dados &eacute; inv&aacute;lida');

        try
        {
            $db = $this->get('db');

            // Reconnect in case the database was created just now

            $db->reconnect();

            $tables = $db->getTables();

            $expectedTables = array(

                UserModel::repo()   ->getTableName(),
                MessageModel::repo()->getTableName(),
                DataModel::repo('') ->getTableName()
            );
        }
        catch(Exception $ex)
        {
            $errors['message'] = 'Exce&ccedil;&atilde;o da base de dados: ' . $ex->getMessage();

            return $errors;
        }
        
        if(count(array_diff($expectedTables, $tables)) !== 0)
        {
            $errors['message'] = 'Estrutura inv&aacute;lida de tabelas, tabela atual: ' . join(', ', $tables) . ', tabelas esperadas: ' . join(', ', $expectedTables);

            return $errors;
        }

        return array();
    }
    
    private function clearEmpty($array)
    {
        // Clear empty entries
        
        $keys = array_keys($array);
        
        for($i = count($array) - 1; $i >= 0; $i--)
        {
            $key = $keys[$i];

            if(is_array($array[$key]))
            {
                $array[$key] = $this->clearEmpty($array[$key]);
            }
            
            if(empty($array[$key]))
            {
                unset($array[$key]);
            }
        }
        
        return $array;
    }
}

?>
